#ifndef IDA_H
#define IDA_H


#include "expr.h"	/* for 'Expr *' in 'Exprlist' */
#include "symbol.h"	/* for 'Symbol *' in 'Exprlist' */
#include "typedef.h"	/* for 'int32' in 'Idref' */

/*{{{  typedef (Idref, Exprlist)*/
#ifdef V_IDA

typedef struct Idref {
	int  lineno;
	int32  ref;
	char  *strval;	/* to store the char repn of an actual arg */
	struct Idref  *next;
} Idref;

#endif

/*--------------------------------------------------------------*/
/* Structure 'Exprlist' is used to create a stack of 'expression*/
/* lists' (e.g. procedure argument lists, array subscript lists,*/
/* etc) that are currently being parsed.			*/
/*--------------------------------------------------------------*/
typedef struct Exprlist {
	struct Symbol *sp;		/* the procedure or array to which
					         the list belongs */
	int            lineno;		/* lineno on which the ref starts */
	int            nexpr;		/* # of exprs currently in the list */
	struct Expr   *last_expr;	/* pointer to the current last expr
					         in the list*/
	struct Exprlist  *next;
} Exprlist;
/*}}}*/

#ifdef V_IDA

/*{{{  #defines (IDA_...)*/
/*--------------------------------------------------------------*/
/* Flags for the 'ref' field of an 'Idref' record.		*/
/* They occupy the highest bits, as the lowest bits may store	*/
/* # (actual arg) (for IDA_ACTUAL).				*/
/*--------------------------------------------------------------*/

#define  IDA_READ		0x10000
#define  IDA_WRITE		0x20000
#define  IDA_ACTUAL		0x40000		/* a var passed to a proc;
						   accompanied by #(arg) */
#define  IDA_PROC_ACTUAL	0x80000		/* a procedure passed to
						   a proc, with #(narg) */
#define  IDA_PROC_REF		0x100000

/*--------------------------------------*/
/* Actual arguments of a proc ref...	*/
/*--------------------------------------*/
#define  IDA_ARG_VAR		0x200000	/* actual arg = a variable */
#define  IDA_ARG_PROC		0x400000	/* actual arg = proc name */
#define  IDA_ARG_ALT_RET	0x800000	/* actual arg = alt return */

#define  IDA_ARG_INTEGER	0x1000000	/* actual arg = int expr */
#define  IDA_ARG_REAL		0x2000000	/*    "    "    real  "  */
#define  IDA_ARG_DP		0x4000000	/*    "    "    dp    "  */
#define  IDA_ARG_COMPLEX	0x8000000	/*    "    "    complex  */
#define  IDA_ARG_LOGICAL	0x10000000	/*    "    "    logical  */
#define  IDA_ARG_CHAR		0x20000000	/*    "    "    char  "  */
#define  IDA_ARG_DERIVED	0x40000000	/*    "    "    derived  */
#define  IDA_ARG_NONE		0x80000000	/*    "    "    unknown  */

#define  IDA_REF_MASK	(IDA_READ | IDA_WRITE | IDA_ACTUAL |\
			 IDA_PROC_ACTUAL | IDA_PROC_REF |\
			 IDA_ARG_VAR | IDA_ARG_PROC | IDA_ARG_ALT_RET |\
			 IDA_ARG_INTEGER | IDA_ARG_REAL | IDA_ARG_DP |\
			 IDA_ARG_COMPLEX | IDA_ARG_LOGICAL |\
			 IDA_ARG_CHAR | IDA_ARG_DERIVED | IDA_ARG_NONE)

#define  IDA_NARG_MASK	(~(IDA_REF_MASK))


#define  IDA_ARG_MASK	(IDA_ARG_VAR | IDA_ARG_PROC | IDA_ARG_ALT_RET |\
			 IDA_ARG_INTEGER | IDA_ARG_REAL | IDA_ARG_DP |\
			 IDA_ARG_COMPLEX | IDA_ARG_LOGICAL | IDA_ARG_CHAR |\
			 IDA_ARG_DERIVED | IDA_ARG_NONE)
/*}}}*/

#else

/*{{{  #defines (replaces proc names by empty string!)*/
#define  install_ida_ref(x,y)			/* empty */
#define  convert_ida_read_ref_to_actual(x)	/* empty */
#define  convert_intrin_actual_refs_to_read(x)	/* empty */
#define  install_ida_proc_actual_ref(x)		/* empty */
#define  install_ida_proc_ref(x)		/* empty */
#define  write_ida_info()			/* empty */
#define  delete_ida_recs(x)			/* empty */
/*}}}*/

#endif

extern Exprlist  *list_stack;	/*  used in 'yyparse()', 'yyerror()',	*/
				/* 'new_exprlist_ref()', 'remove_exprlist_ref()'*/

#endif
